<?php

/**
 * The public-facing functionality of Visidea.
 *
 * @link       https://visidea.ai
 * @since      1.0.0
 *
 * @package    Visidea
 * @subpackage Visidea/public
 */

class Visidea_Public {

  /**
   * The ID of Visidea.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of Visidea.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of the plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {

  	$this->plugin_name = $plugin_name;
  	$this->version = $version;

  }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since 1.0.0
     */
    public function enqueue_styles() {

        wp_register_style('visidea_slick', plugin_dir_url( __FILE__ ) . 'css/slick.css', array(), $this->version, 'all');
        wp_register_style('visidea_slick_theme', plugin_dir_url( __FILE__ ) . 'css/slick-theme.css', array(), $this->version, 'all');

        wp_enqueue_style('visidea_slick');
        wp_enqueue_style('visidea_slick_theme');

    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since 1.0.0
     */
    public function enqueue_scripts() {

        wp_register_script('visidea_slick', plugin_dir_url( __FILE__ ) . 'js/slick.min.js', array('jquery'), $this->version, false);
        wp_register_script('visidea_api_client', plugin_dir_url( __FILE__ ) . 'js/visidea-api-client.min.js', array('jquery'), $this->version, false);
        wp_register_script('visidea_client', plugin_dir_url( __FILE__ ) . 'js/visidea-client.js', array('jquery'), $this->version, false);

        wp_enqueue_script('visidea_slick');
        wp_enqueue_script('visidea_api_client');
        wp_enqueue_script('visidea_client');

        // Initialize VisideaWoocommerce object
        $options = get_option('visidea_plugin_options');
        $visidea_public_token = $options['public_token'];
        $visidea_database = str_replace('.','_',$options['database']);
        wp_add_inline_script('visidea_client', 'var visideaWoocommerce = new VisideaWoocommerce("'.esc_js($visidea_database).'","'.esc_js($visidea_public_token).'");');

    }

  /**
   * Action to execute when an order is placed.
   * Append to the interaction file (filename defined in activator)
   * the string user_id;purchase;item_id for each item in the
   * placed order.
   *
   * @since 1.0.0
   * @param int $order_id identificator of the placed order
   */
  public function callback_order_placed($order_id) {

    if (!$order_id)
      return;

    // Allow code execution only once
    if(!get_post_meta($order_id, '_thankyou_action_done', true)) {

      // Get handle to Visidea API
      $options = get_option('visidea_plugin_options');
      $visidea_public_token = $options['public_token'];
      $visidea_database = $options['database'];

      // Get an instance of the WC_Order object
      $order = wc_get_order( $order_id );

      // Get user ID
      $user_id = $order->get_user_id();

      $items_array = array();

      // Loop through order items and write them in the opened file
      foreach ($order->get_items() as $item_id => $item) {

        // Get product handle
        $product = $item->get_product();

        // Get the product ID
        $product_id = $product->get_id();

        // Append id to array to pass to JS
        array_push($items_array, $product_id);

      }

      // Flag the action as done (to avoid repetitions on reload for example)
      $order->update_meta_data( '_thankyou_action_done', true );
      $order->save();

      // Use VisideaWoocommerce to send purchase messages
      echo "<script> visideaWoocommerce.interaction('order', 'purchase', '".esc_js($user_id)."', ".esc_js(json_encode($items_array))."); </script>";
    }

  }


  /**
   *    Action to be performed when a product is addedd to a
   *    user's cart. Append the string item_id;cart;user_id
   *    to the file specified in the activator.
   *
   *    @since 1.0.0
   *    @param int      $cart_item_key          cart id
   *    @param int      $product_id             product id
   *    @param int      $quantity
   *    @param int      $variation_id           variation of product id
   *    @param int      $variation
   *    @param ?        $cart_item_data
   *
   *    // yeah, I know.
   */
  public function callback_product_added_cart($cart_item_key, $product_id, $quantity,
    $variation_id, $variation, $cart_item_data ) {

    $user_id = get_current_user_id();

    // Get handle to Visidea API
    $options = get_option('visidea_plugin_options');
    $visidea_public_token = $options['public_token'];
    $visidea_database = $options['database'];

    $item_array = [$product_id];

    // Use VisideaWoocommerce to send purchase messages
    echo "<script> visideaWoocommerce.interaction('cart', 'cart', '".esc_js($user_id)."', ".esc_js(json_encode($item_array))."); </script>";

  }


  public function callback_home_viewed() {
      if (is_home()) {
          $user_id = get_current_user_id();

          // echo 'after content';
          $options = get_option('visidea_plugin_options');
          $visidea_public_token = $options['public_token'];
          $visidea_database = $options['database'];

          // Show recommendations
          echo "<script> visideaWoocommerce.recommend('home'); </script>";

      }
  }

  /**
   *  Action to be performed when a product page is loaded.
   *  Append to the file defined in the activator
   *  the string product_id;view;user_id. Since there's no way
   *  to perform this action _only the first time_ it's viewed,
   *  it will be repeated every time the user views a product page.
   */
  public function callback_product_viewed() {
    global $product;
    $product_id = $product->get_id();
    $user_id = get_current_user_id();

    // Get handle to Visidea API
    $options = get_option('visidea_plugin_options');
    $visidea_public_token = $options['public_token'];
    $visidea_database = $options['database'];

    $item_array = [$product_id];
    // Use VisideaWoocommerce to send product view messages
    echo "<script> visideaWoocommerce.interaction('product', 'view', '".esc_js($user_id)."', ".esc_js(json_encode($item_array))."); </script>";

    // Show recommendations
    echo "<script> visideaWoocommerce.recommend('product'); </script>";
  }

  public function callback_recommends_shop_cart() {
    if (is_cart()) {

      $user_id = get_current_user_id();

      // Get handle to Visidea API
      $options = get_option('visidea_plugin_options');
      $visidea_public_token = $options['public_token'];
      $visidea_database = $options['database'];

      // Use VisideaWoocommerce to send cart messages
      echo "<script> visideaWoocommerce.interaction('cart', 'cart', '".esc_js($user_id)."',''); </script>";

      // Show recommendations
      echo "<script> visideaWoocommerce.recommend('cart'); </script>";
    }
  }
}
